/*
 * Copyright (C) 2006 INDT
 *  Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <stdio.h>
#include <stdlib.h>

#include <libdmapsharing/daap-connection.h>
#include <libdmapsharing/dpap-connection.h>
#include <libdmapsharing/dmap-mdns-browser.h>
#include <libdmapsharing/dmap-item.h>

enum {
    DAAP,
    DPAP
};

static void loaded_cb (DMAPItem *item, gpointer user_data);

static GMainLoop *loop;
static guint conn_type = DAAP;

static void
items_retrieved_cb (DMAPItem *item,
                    const GSList *items)
{
    const GSList *l;
    gchar *name, *uri;
    gboolean is_folder;

    g_debug ("%d items retrieved", g_slist_length ((GSList*) items));

    for (l = items; l; l = l->next) {
        DMAPItem *item = (DMAPItem *) l->data;

        g_object_get (G_OBJECT (item),
                "name", &name,
                "is_folder", &is_folder, NULL);
        if (is_folder) {
            g_signal_connect (G_OBJECT (item),
                              "loaded",
                              G_CALLBACK (loaded_cb),
                              NULL);

            g_debug ("retrieving items from item %s", name);
            dmap_item_retrieve_items (item, FALSE);
        }
        else {
            g_object_get (G_OBJECT (item), "uri", &uri, NULL);
            g_print ("\n\ttitle: %s\n"
                     "\turi: %s\n",
                     name,
                     uri);
            g_free (uri);
        }
        g_free (name);
    }
}

static void
loaded_cb (DMAPItem *item, gpointer user_data)
{
    items_retrieved_cb (item, dmap_item_get_items (item));
}

/*
static gboolean
disconnect_cb (DMAPConnection *conn)
{
    dmap_connection_disconnect (conn);
    g_object_unref (conn);
    g_main_loop_quit (loop);
    return FALSE;
}
*/

static void
connected_cb (DMAPConnection *conn)
{
    DMAPItem *root_item;

    g_debug ("connected");

    root_item = dmap_connection_get_root_item (conn);
    dmap_item_retrieve_items (root_item, FALSE);
    g_object_unref (root_item);
    /* g_timeout_add (10000, (GSourceFunc) disconnect_cb, conn); */
}

static void
service_added_cb (DMAPMdnsBrowser *browser,
                  DMAPMdnsBrowserService *service,
                  gpointer user_data)
{
    DMAPConnection *conn;
    DMAPItem *root_item;

    g_debug ("service added %s:%s:%s:%d (%s)",
             service->service_name,
             service->name,
             service->host,
             service->port,
             service->password_protected ? "protected" : "not protected");

    if (conn_type == DAAP)
        conn = DMAP_CONNECTION (daap_connection_new (service->name, service->host, service->port, NULL));
    else
        conn = DMAP_CONNECTION (dpap_connection_new (service->name, service->host, service->port, NULL));
    g_signal_connect (G_OBJECT (conn),
                      "connected",
                      G_CALLBACK (connected_cb),
                      NULL);
    root_item = dmap_connection_get_root_item (conn);
    g_signal_connect (G_OBJECT (root_item),
                      "loaded",
                      G_CALLBACK (loaded_cb),
                      NULL);
    g_object_unref (root_item);
    dmap_connection_connect (DMAP_CONNECTION (conn));
}

int main(int argc, char **argv)
{
    DMAPMdnsBrowser *browser;
    GError *error = NULL;

    if (argc == 2)
        conn_type = atoi (argv[1]);

    g_type_init ();
    g_thread_init (NULL);

    loop = g_main_loop_new (NULL, FALSE);

    if (conn_type == DAAP)
        browser = dmap_mdns_browser_new (DMAP_MDNS_BROWSER_SERVICE_TYPE_DAAP);
    else
        browser = dmap_mdns_browser_new (DMAP_MDNS_BROWSER_SERVICE_TYPE_DPAP);
    g_signal_connect (G_OBJECT (browser),
                      "service-added",
                      G_CALLBACK (service_added_cb),
                      NULL);
    g_debug ("starting mdns browser");
    dmap_mdns_browser_start (browser, &error);
    if (error) {
        g_warning ("error starting browser. code: %d message: %s",
                error->code,
                error->message);
        return 1;
    }

    g_main_loop_run (loop);

    return 0;
}
