/*
 * Database record class for DPAP sharing
 *
 * Copyright (C) 2008 W. Michael Petullo <mike@flyn.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "test-dpap-record.h"

struct TestDPAPRecordPrivate {
	gint filesize;
	gint largefilesize;
	gint pixelheight;
	gint pixelwidth;
	gint rating;
	gint creationdate;
	const char *location;
	const char *title;
	const char *aspectratio;
	const char *filename;
	const char *format;
	const char *comments;
	unsigned char *thumbnail;
};

const char *
test_dpap_record_get_location (DPAPRecord *record)
{
	return TEST_DPAP_RECORD (record)->priv->location;
}

const char *
test_dpap_record_get_title (DPAPRecord *record)
{
	return TEST_DPAP_RECORD (record)->priv->title;
}

const char *
test_dpap_record_get_aspectratio (DPAPRecord *record)
{
	return TEST_DPAP_RECORD (record)->priv->aspectratio;
}

const char *
test_dpap_record_get_filename (DPAPRecord *record)
{
	return TEST_DPAP_RECORD (record)->priv->filename;
}

const char *
test_dpap_record_get_format (DPAPRecord *record)
{
	return TEST_DPAP_RECORD (record)->priv->format;
}

const char *
test_dpap_record_get_comments (DPAPRecord *record)
{
	return TEST_DPAP_RECORD (record)->priv->comments;
}

gint
test_dpap_record_get_filesize (DPAPRecord *record)
{
	return TEST_DPAP_RECORD (record)->priv->filesize;
}

gint
test_dpap_record_get_largefilesize (DPAPRecord *record)
{
	return TEST_DPAP_RECORD (record)->priv->largefilesize;
}

gint
test_dpap_record_get_pixelheight (DPAPRecord *record)
{
	return TEST_DPAP_RECORD (record)->priv->pixelheight;
}

gint
test_dpap_record_get_pixelwidth (DPAPRecord *record)
{
	return TEST_DPAP_RECORD (record)->priv->pixelwidth;
}

gint
test_dpap_record_get_rating (DPAPRecord *record)
{
	return TEST_DPAP_RECORD (record)->priv->rating;
}

gint
test_dpap_record_get_creationdate (DPAPRecord *record)
{
	return TEST_DPAP_RECORD (record)->priv->creationdate;
}

unsigned char *
test_dpap_record_get_thumbnail (DPAPRecord *record, size_t *size)
{
	*size = TEST_DPAP_RECORD (record)->priv->filesize;
	return TEST_DPAP_RECORD (record)->priv->thumbnail;
}

GInputStream *test_dpap_record_read (DPAPRecord *record, GError **error)
{
	GFile *file;
	GInputStream *stream;

	file = g_file_new_for_uri (TEST_DPAP_RECORD (record)->priv->location);
	stream = G_INPUT_STREAM (g_file_read (file, NULL, error));

	g_object_unref (file);

	return stream;
}



static void
test_dpap_record_init (TestDPAPRecord *record)
{
	record->priv = TEST_DPAP_RECORD_GET_PRIVATE (record);
}

static void
test_dpap_record_class_init (TestDPAPRecordClass *klass)
{
	g_type_class_add_private (klass, sizeof (TestDPAPRecordPrivate));
}

static void
test_dpap_record_interface_init (gpointer iface, gpointer data)
{
	DPAPRecordInterface *dpap_record = iface;

	g_assert (G_TYPE_FROM_INTERFACE (dpap_record) == TYPE_DPAP_RECORD);

	dpap_record->get_location = test_dpap_record_get_location;
	dpap_record->get_title = test_dpap_record_get_title;
	dpap_record->get_format = test_dpap_record_get_format;
	dpap_record->get_filesize = test_dpap_record_get_filesize;
	dpap_record->get_rating = test_dpap_record_get_rating;
	dpap_record->get_aspectratio = test_dpap_record_get_aspectratio;
	dpap_record->get_filename = test_dpap_record_get_filename;
	dpap_record->get_creationdate = test_dpap_record_get_creationdate;
	dpap_record->get_largefilesize = test_dpap_record_get_largefilesize;
	dpap_record->get_pixelheight = test_dpap_record_get_pixelheight;
	dpap_record->get_pixelwidth = test_dpap_record_get_pixelwidth;
	dpap_record->get_comments = test_dpap_record_get_comments;
	dpap_record->get_thumbnail = test_dpap_record_get_thumbnail;
	dpap_record->read = test_dpap_record_read;
}

G_DEFINE_TYPE_WITH_CODE (TestDPAPRecord, test_dpap_record, G_TYPE_OBJECT, 
			 G_IMPLEMENT_INTERFACE (TYPE_DPAP_RECORD,
					       test_dpap_record_interface_init))

TestDPAPRecord *
test_dpap_record_new (void)
{
	unsigned char *thumbnail;
	GError *error;
	gchar *path;
	gsize size;
	TestDPAPRecord *record;

	record = TEST_DPAP_RECORD (g_object_new (TYPE_TEST_DPAP_RECORD, NULL));

	/* Must be a URI. */
	record->priv->location = g_strdup_printf ("file://%s/media/test.jpeg",
						  g_get_current_dir ());

	record->priv->title = "Title of Photograph";

	/* Width / Height as a string. */
	record->priv->aspectratio = "1.333";

	record->priv->filename = g_basename (test_dpap_record_get_location (DPAP_RECORD (record)));

	record->priv->format = "JPEG";

	record->priv->comments = "Test comments about an image.";

	record->priv->largefilesize = 13953;

	record->priv->pixelheight = 480;

	record->priv->pixelwidth = 640;

	record->priv->rating = 5;

	record->priv->creationdate = 0;

	/* Normally, this data is scaled down to thumbnail size. I have not
	 * done this here because I don't want any external dependencies.
	 * In many cases the ImageMagick library is a good solution.
	 */
	path = g_strdup_printf ("%s/media/test.jpeg", g_get_current_dir ());
	g_file_get_contents (path, (gchar **) &thumbnail, &size, &error);
	g_free (path);
	record->priv->filesize = size;
	record->priv->thumbnail = thumbnail;

	return record;
}
